@@ -23,7 +23,6 @@ from __future__ import division, print_function, absolute_import
 import os
 import ast
 import sys
-import time
 import zlib
 import base64
 import random as rnd_module
@@ -49,13 +48,14 @@ from hypothesis.version import __version__
 from hypothesis._settings import Phase, Verbosity, HealthCheck, \
     PrintSettings
 from hypothesis._settings import settings as Settings
-from hypothesis._settings import note_deprecation
+from hypothesis._settings import local_settings, note_deprecation
 from hypothesis.executors import new_style_executor
 from hypothesis.reporting import report, verbose_report, current_verbosity
 from hypothesis.statistics import note_engine_for_statistics
 from hypothesis.internal.compat import ceil, hbytes, qualname, \
     str_to_bytes, benchmark_time, get_type_hints, getfullargspec, \
     int_from_bytes, encoded_filepath, bad_django_TestCase
+from hypothesis.internal.entropy import deterministic_PRNG
 from hypothesis.internal.coverage import IN_COVERAGE_TESTS
 from hypothesis.utils.conventions import infer, not_set
 from hypothesis.internal.escalation import is_hypothesis_file, \
@@ -384,7 +384,25 @@ def skip_exceptions_to_reraise():
     return tuple(sorted(exceptions, key=str))
 
 
-exceptions_to_reraise = skip_exceptions_to_reraise()
+EXCEPTIONS_TO_RERAISE = skip_exceptions_to_reraise()
+
+
+def failure_exceptions_to_catch():
+    """Return a tuple of exceptions meaning 'this test has failed', to catch.
+
+    This is intended to cover most common test runners; if you would
+    like another to be added please open an issue or pull request.
+    """
+    exceptions = [Exception]
+    try:  # pragma: no cover
+        from _pytest.outcomes import Failed
+        exceptions.append(Failed)
+    except ImportError:
+        pass
+    return tuple(exceptions)
+
+
+EXCEPTIONS_TO_FAIL = failure_exceptions_to_catch()
 
 
 def new_given_argspec(original_argspec, generator_kwargs):
@@ -552,10 +570,10 @@ class StateForActualGivenExecution(object):
         def run(data):
             if not hasattr(data, 'can_reproduce_example_from_repr'):
                 data.can_reproduce_example_from_repr = True
-            with self.settings:
+            with local_settings(self.settings):
                 with BuildContext(data, is_final=is_final):
-                    rnd_module.seed(0)
-                    args, kwargs = data.draw(self.search_strategy)
+                    with deterministic_PRNG():
+                        args, kwargs = data.draw(self.search_strategy)
                     if expected_failure is not None:
                         text_repr[0] = arg_string(test, args, kwargs)
 
@@ -573,11 +591,13 @@ class StateForActualGivenExecution(object):
                                 test.__name__, arg_string(test, args, kwargs)))
 
                     if self.collector is None or not collect:
-                        return test(*args, **kwargs)
+                        with deterministic_PRNG():
+                            return test(*args, **kwargs)
                     else:  # pragma: no cover
                         try:
                             self.collector.start()
-                            return test(*args, **kwargs)
+                            with deterministic_PRNG():
+                                return test(*args, **kwargs)
                         finally:
                             self.collector.stop()
 
@@ -690,7 +710,7 @@ class StateForActualGivenExecution(object):
         except (
             HypothesisDeprecationWarning, FailedHealthCheck,
             StopTest,
-        ) + exceptions_to_reraise:
+        ) + EXCEPTIONS_TO_RERAISE:
             raise
         except Exception as e:
             escalate_hypothesis_internal_error()
@@ -712,7 +732,7 @@ class StateForActualGivenExecution(object):
             database_key = str_to_bytes(fully_qualified_name(self.test))
         else:
             database_key = None
-        self.start_time = time.time()
+        self.start_time = benchmark_time()
         global in_given
         runner = ConjectureRunner(
             self.evaluate_test_data,
@@ -734,7 +754,7 @@ class StateForActualGivenExecution(object):
                 self.used_examples_from_database = \
                     runner.used_examples_from_database
         note_engine_for_statistics(runner)
-        run_time = time.time() - self.start_time
+        run_time = benchmark_time() - self.start_time
 
         self.used_examples_from_database = runner.used_examples_from_database
 
@@ -891,6 +911,25 @@ def fake_subTest(self, msg=None, **__):
     yield
 
 
+@attr.s()
+class HypothesisHandle(object):
+    """This object is provided as the .hypothesis attribute on @given tests.
+
+    Downstream users can reassign its attributes to insert custom logic into
+    the execution of each case, for example by converting an async into a
+    sync function.
+
+    This must be an attribute of an attribute, because reassignment of a
+    first-level attribute would not be visible to Hypothesis if the function
+    had been decorated before the assignment.
+
+    See https://github.com/HypothesisWorks/hypothesis/issues/1257 for more
+    information.
+    """
+
+    inner_test = attr.ib()
+
+
 def given(
     *given_arguments,  # type: Union[SearchStrategy, InferType]
     **given_kwargs  # type: Union[SearchStrategy, InferType]
@@ -936,6 +975,8 @@ def given(
             # Tell pytest to omit the body of this function from tracebacks
             __tracebackhide__ = True
 
+            test = wrapped_test.hypothesis.inner_test
+
             if getattr(test, 'is_hypothesis_test', False):
                 note_deprecation(
                     'You have applied @given to a test more than once. In '
@@ -1077,6 +1118,7 @@ def given(
         wrapped_test._hypothesis_internal_use_reproduce_failure = getattr(
             test, '_hypothesis_internal_use_reproduce_failure', None
         )
+        wrapped_test.hypothesis = HypothesisHandle(test)
         return wrapped_test
     return run_test_with_generator
 
@@ -1091,10 +1133,8 @@ def find(
     # type: (...) -> Any
     """Returns the minimal example from the given strategy ``specifier`` that
     matches the predicate function ``condition``."""
-    settings = settings or Settings(
-        max_examples=2000,
-        max_shrinks=2000,
-    )
+    if settings is None:
+        settings = Settings(max_examples=2000)
     settings = Settings(settings, suppress_health_check=HealthCheck.all())
 
     if database_key is None and settings.database is not None:
@@ -1118,9 +1158,10 @@ def find(
         with BuildContext(data):
             try:
                 data.is_find = True
-                result = data.draw(search)
-                data.note(result)
-                success = condition(result)
+                with deterministic_PRNG():
+                    result = data.draw(search)
+                    data.note(result)
+                    success = condition(result)
             except UnsatisfiedAssumption:
                 data.mark_invalid()
 
@@ -1145,19 +1186,20 @@ def find(
                     last_data[0] = data
         if success and not data.frozen:
             data.mark_interesting()
-    start = time.time()
+    start = benchmark_time()
     runner = ConjectureRunner(
         template_condition, settings=settings, random=random,
         database_key=database_key,
     )
     runner.run()
     note_engine_for_statistics(runner)
-    run_time = time.time() - start
+    run_time = benchmark_time() - start
     if runner.interesting_examples:
         data = ConjectureData.for_buffer(
             list(runner.interesting_examples.values())[0].buffer)
         with BuildContext(data):
-            return data.draw(search)
+            with deterministic_PRNG():
+                return data.draw(search)
     if runner.valid_examples == 0 and (
         runner.exit_reason != ExitReason.finished
     ):
