@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2017 David R. MacIver
+# Most of this work is copyright (C) 2013-2018 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -19,23 +19,25 @@
 
 from __future__ import division, print_function, absolute_import
 
-import os
 import re
 import sys
 import math
+import time
+import array
 import codecs
 import platform
 import importlib
 from base64 import b64encode
-from decimal import Context, Decimal, Inexact
-from hashlib import sha1
 from collections import namedtuple
 
 try:
     from collections import OrderedDict, Counter
-except ImportError:  # pragma: no cover
-    from ordereddict import OrderedDict
-    from counter import Counter
+except ImportError:
+    from ordereddict import OrderedDict  # type: ignore
+    from counter import Counter  # type: ignore
+
+if False:
+    from typing import Type  # noqa
 
 
 PY2 = sys.version_info[0] == 2
@@ -55,10 +57,6 @@ def bit_length(n):
     return n.bit_length()
 
 
-def float_to_decimal(f):
-    return Decimal(f)
-
-
 if PY3:
     def str_to_bytes(s):
         return s.encode(a_good_encoding())
@@ -72,7 +70,6 @@ if PY3:
     ARG_NAME_ATTRIBUTE = 'arg'
     integer_types = (int,)
     hunichr = chr
-    from functools import reduce
 
     def unicode_safe_repr(x):
         return repr(x)
@@ -97,15 +94,9 @@ def quiet_raise(exc):
     def int_to_bytes(i, size):
         return i.to_bytes(size, 'big')
 
-    def bytes_from_list(ls):
-        return bytes(ls)
-
     def to_bytes_sequence(ls):
         return bytes(ls)
 
-    def zero_byte_sequence(n):
-        return bytes(n)
-
     def int_to_byte(i):
         return bytes([i])
 
@@ -114,7 +105,8 @@ def quiet_raise(exc):
     struct_pack = struct.pack
     struct_unpack = struct.unpack
 
-    from time import monotonic as benchmark_time
+    def benchmark_time():
+        return time.monotonic()
 else:
     import struct
 
@@ -128,15 +120,14 @@ else:
         def struct_unpack(fmt, string):
             return struct.unpack(fmt, str(string))
 
-    def zero_byte_sequence(n):
-        return hbytes(b'\0' * n)
-
     def int_from_bytes(data):
-        assert isinstance(data, bytearray)
         if CAN_UNPACK_BYTE_ARRAY:
             unpackable_data = data
-        else:
+        elif isinstance(data, bytearray):
             unpackable_data = bytes(data)
+        else:
+            unpackable_data = data
+        assert isinstance(data, (bytes, bytearray))
         result = 0
         i = 0
         while i + 4 <= len(data):
@@ -153,19 +144,18 @@ else:
         assert i >= 0
         result = bytearray(size)
         j = size - 1
+        arg = i
         while i and j >= 0:
             result[j] = i & 255
             i >>= 8
             j -= 1
         if i:
-            raise OverflowError('int too big to convert')
+            raise OverflowError('i=%r cannot be represented in %r bytes'
+                                % (arg, size))
         return hbytes(result)
 
     int_to_byte = chr
 
-    def bytes_from_list(ls):
-        return hbytes(bytearray(ls))
-
     def to_bytes_sequence(ls):
         return bytearray(ls)
 
@@ -227,7 +217,6 @@ else:
     ARG_NAME_ATTRIBUTE = 'id'
     integer_types = (int, long)
     hunichr = unichr
-    reduce = reduce
 
     def escape_unicode_characters(s):
         return codecs.encode(s, 'string_escape')
@@ -240,7 +229,8 @@ else:
     def quiet_raise(exc):
         raise exc
 
-    from time import time as benchmark_time
+    def benchmark_time():
+        return time.time()
 
 
 # coverage mixes unicode and str filepaths on Python 2, which causes us
@@ -294,22 +284,6 @@ if PY2:
 else:
     from inspect import getfullargspec, FullArgSpec
 
-    if sys.version_info[:2] == (3, 5):
-        # silence deprecation warnings on Python 3.5
-        # (un-deprecated in 3.6 to allow single-source 2/3 code like this)
-        def silence_warnings(func):
-            import warnings
-            import functools
-
-            @functools.wraps(func)
-            def inner(*args, **kwargs):
-                with warnings.catch_warnings():
-                    warnings.simplefilter('ignore', DeprecationWarning)
-                    return func(*args, **kwargs)
-            return inner
-
-        getfullargspec = silence_warnings(getfullargspec)
-
 
 if sys.version_info[:2] < (3, 6):
     def get_type_hints(thing):
@@ -380,7 +354,6 @@ def update_code_location(code, newfile, newlineno):
     hack that I've done purely for vanity, and if you're reading this
     code you're probably here because it's broken something and now
     you're angry at me. Sorry.
-
     """
     unpacked = [
         getattr(code, name) for name in CODE_FIELD_ORDER
@@ -442,7 +415,7 @@ class compatbytes(bytearray):
         else:
             return r
 
-    __setitem__ = None
+    __setitem__ = None  # type: ignore
 
     def join(self, parts):
         result = bytearray()
@@ -499,7 +472,7 @@ def implements_iterator(it):
 
 
 if PY3:
-    FileNotFoundError = FileNotFoundError
+    FileNotFoundError = FileNotFoundError  # type: Type[IOError]
 else:
     FileNotFoundError = IOError
 
@@ -507,7 +480,7 @@ else:
 # an existing file where you're not allowed to. This is rather less consistent
 # between versions than might be hoped.
 if PY3:
-    FileExistsError = FileExistsError
+    FileExistsError = FileExistsError  # type: Type[IOError]
 
 elif WINDOWS:
     FileExistsError = WindowsError
@@ -549,3 +522,18 @@ if PY2:
         return hbytes(base(s))
 else:
     from base64 import b64decode
+
+
+try:
+    from django.test import TransactionTestCase
+    from hypothesis.extra.django import HypothesisTestCase
+
+    def bad_django_TestCase(runner):
+        if runner is None:
+            return False
+        return isinstance(runner, TransactionTestCase) and \
+            not isinstance(runner, HypothesisTestCase)
+except Exception:
+    # Can't use ImportError, because of e.g. Django config errors
+    def bad_django_TestCase(runner):
+        return False
