@@ -28,20 +28,21 @@ from __future__ import division, print_function, absolute_import
 
 import inspect
 import traceback
+from copy import copy
 from unittest import TestCase
 
 import attr
 
 import hypothesis.internal.conjecture.utils as cu
-from hypothesis.core import find
+from hypothesis.core import EXCEPTIONS_TO_FAIL, find
 from hypothesis.errors import Flaky, NoSuchExample, InvalidDefinition, \
     HypothesisException
 from hypothesis.control import BuildContext
 from hypothesis._settings import Verbosity
 from hypothesis._settings import settings as Settings
 from hypothesis.reporting import report, verbose_report, current_verbosity
-from hypothesis.strategies import just, lists, builds, one_of, runner, \
-    tuples, integers, fixed_dictionaries
+from hypothesis.strategies import just, one_of, runner, tuples, \
+    fixed_dictionaries
 from hypothesis.vendor.pretty import CUnicodeIO, RepresentationPrinter
 from hypothesis.internal.reflection import proxies, nicerepr
 from hypothesis.internal.conjecture.data import StopTest
@@ -51,6 +52,9 @@ from hypothesis.searchstrategy.strategies import SearchStrategy
 
 STATE_MACHINE_RUN_LABEL = calc_label_from_name('another state machine step')
 
+if False:
+    from typing import Any, Dict, List, Text  # noqa
+
 
 class TestCaseProperty(object):  # pragma: no cover
 
@@ -73,7 +77,7 @@ def find_breaking_runner(state_machine_factory, settings=None):
             return False
         except HypothesisException:
             raise
-        except Exception:
+        except EXCEPTIONS_TO_FAIL:
             verbose_report(traceback.format_exc)
             return True
     if settings is None:
@@ -136,6 +140,8 @@ class GenericStateMachine(object):
     sequence of example choices demonstrating that.
     """
 
+    find_breaking_runner = None  # type: classmethod
+
     def steps(self):
         """Return a SearchStrategy instance the defines the available next
         steps."""
@@ -145,6 +151,18 @@ class GenericStateMachine(object):
         """Execute a step that has been previously drawn from self.steps()"""
         raise NotImplementedError(u'%r.execute_step()' % (self,))
 
+    def print_start(self):
+        """Called right at the start of printing.
+
+        By default does nothing.
+        """
+
+    def print_end(self):
+        """Called right at the end of printing.
+
+        By default does nothing.
+        """
+
     def print_step(self, step):
         """Print a step to the current reporter.
 
@@ -157,7 +175,7 @@ class GenericStateMachine(object):
         """Called after a run has finished executing to clean up any necessary
         state.
 
-        Does nothing by default
+        Does nothing by default.
         """
         pass
 
@@ -165,7 +183,7 @@ class GenericStateMachine(object):
         """Called after initializing and after executing each step."""
         pass
 
-    _test_case_cache = {}
+    _test_case_cache = {}  # type: dict
 
     TestCase = TestCaseProperty()
 
@@ -177,9 +195,7 @@ class GenericStateMachine(object):
             pass
 
         class StateMachineTestCase(TestCase):
-            settings = Settings(
-                min_satisfying_examples=1
-            )
+            settings = Settings()
 
         # We define this outside of the class and assign it because you can't
         # assign attributes to instance method values in Python 2
@@ -221,27 +237,25 @@ class StateMachineRunner(object):
             print_steps = current_verbosity() >= Verbosity.debug
         self.data.hypothesis_runner = state_machine
 
-        stopping_value = 1 - 1.0 / (1 + self.n_steps * 0.5)
+        should_continue = cu.many(
+            self.data, min_size=1, max_size=self.n_steps,
+            average_size=self.n_steps,
+        )
+
         try:
+            if print_steps:
+                state_machine.print_start()
             state_machine.check_invariants()
 
-            steps = 0
-            while True:
-                if steps >= self.n_steps:
-                    stopping_value = 0
-                self.data.start_example(STATE_MACHINE_RUN_LABEL)
-                if not cu.biased_coin(self.data, stopping_value):
-                    self.data.stop_example()
-                    break
-                assert steps < self.n_steps
+            while should_continue.more():
                 value = self.data.draw(state_machine.steps())
-                steps += 1
                 if print_steps:
                     state_machine.print_step(value)
                 state_machine.execute_step(value)
-                self.data.stop_example()
                 state_machine.check_invariants()
         finally:
+            if print_steps:
+                state_machine.print_end()
             state_machine.teardown()
 
 
@@ -265,8 +279,7 @@ class Rule(object):
 self_strategy = runner()
 
 
-class Bundle(SearchStrategy):
-
+class BundleReferenceStrategy(SearchStrategy):
     def __init__(self, name):
         self.name = name
 
@@ -275,12 +288,23 @@ class Bundle(SearchStrategy):
         bundle = machine.bundle(self.name)
         if not bundle:
             data.mark_invalid()
-        reference = bundle.pop()
-        bundle.insert(integer_range(data, 0, len(bundle)), reference)
+        return bundle[integer_range(data, 0, len(bundle) - 1)]
+
+
+class Bundle(SearchStrategy):
+
+    def __init__(self, name):
+        self.name = name
+        self.__reference_strategy = BundleReferenceStrategy(name)
+
+    def do_draw(self, data):
+        machine = data.draw(self_strategy)
+        reference = data.draw(self.__reference_strategy)
         return machine.names_to_values[reference.name]
 
 
 RULE_MARKER = u'hypothesis_stateful_rule'
+INITIALIZE_RULE_MARKER = u'hypothesis_stateful_initialize_rule'
 PRECONDITION_MARKER = u'hypothesis_stateful_precondition'
 INVARIANT_MARKER = u'hypothesis_stateful_invariant'
 
@@ -308,7 +332,8 @@ def rule(targets=(), target=None, **kwargs):
 
     def accept(f):
         existing_rule = getattr(f, RULE_MARKER, None)
-        if existing_rule is not None:
+        existing_initialize_rule = getattr(f, INITIALIZE_RULE_MARKER, None)
+        if existing_rule is not None or existing_initialize_rule is not None:
             raise InvalidDefinition(
                 'A function cannot be used for two distinct rules. ',
                 Settings.default,
@@ -326,6 +351,49 @@ def rule(targets=(), target=None, **kwargs):
     return accept
 
 
+def initialize(targets=(), target=None, **kwargs):
+    """Decorator for RuleBasedStateMachine.
+
+    An initialize decorator behaves like a rule, but the decorated
+    method is called at most once in a run. All initialize decorated
+    methods will be called before any rule decorated methods, in an
+    arbitrary order.
+    """
+    if target is not None:
+        targets += (target,)
+
+    converted_targets = []
+    for t in targets:
+        while isinstance(t, Bundle):
+            t = t.name
+        converted_targets.append(t)
+
+    def accept(f):
+        existing_rule = getattr(f, RULE_MARKER, None)
+        existing_initialize_rule = getattr(f, INITIALIZE_RULE_MARKER, None)
+        if existing_rule is not None or existing_initialize_rule is not None:
+            raise InvalidDefinition(
+                'A function cannot be used for two distinct rules. ',
+                Settings.default,
+            )
+        precondition = getattr(f, PRECONDITION_MARKER, None)
+        if precondition:
+            raise InvalidDefinition(
+                'An initialization rule cannot have a precondition. ',
+                Settings.default,
+            )
+        rule = Rule(targets=tuple(converted_targets), arguments=kwargs,
+                    function=f, precondition=precondition)
+
+        @proxies(f)
+        def rule_wrapper(*args, **kwargs):
+            return f(*args, **kwargs)
+
+        setattr(rule_wrapper, INITIALIZE_RULE_MARKER, rule)
+        return rule_wrapper
+    return accept
+
+
 @attr.s()
 class VarReference(object):
     name = attr.ib()
@@ -356,6 +424,13 @@ def precondition(precond):
         def precondition_wrapper(*args, **kwargs):
             return f(*args, **kwargs)
 
+        existing_initialize_rule = getattr(f, INITIALIZE_RULE_MARKER, None)
+        if existing_initialize_rule is not None:
+            raise InvalidDefinition(
+                'An initialization rule cannot have a precondition. ',
+                Settings.default,
+            )
+
         rule = getattr(f, RULE_MARKER, None)
         if rule is None:
             setattr(precondition_wrapper, PRECONDITION_MARKER, precond)
@@ -413,12 +488,6 @@ def invariant():
     return accept
 
 
-@attr.s()
-class ShuffleBundle(object):
-    bundle = attr.ib()
-    swaps = attr.ib()
-
-
 class RuleBasedStateMachine(GenericStateMachine):
     """A RuleBasedStateMachine gives you a more structured way to define state
     machines.
@@ -430,22 +499,27 @@ class RuleBasedStateMachine(GenericStateMachine):
     executed.
     """
 
-    _rules_per_class = {}
-    _invariants_per_class = {}
-    _base_rules_per_class = {}
+    _rules_per_class = {}  # type: Dict[type, List[classmethod]]
+    _invariants_per_class = {}  # type: Dict[type, List[classmethod]]
+    _base_rules_per_class = {}  # type: Dict[type, List[classmethod]]
+    _initializers_per_class = {}  # type: Dict[type, List[classmethod]]
+    _base_initializers_per_class = {}  # type: Dict[type, List[classmethod]]
 
     def __init__(self):
         if not self.rules():
             raise InvalidDefinition(u'Type %s defines no rules' % (
                 type(self).__name__,
             ))
-        self.bundles = {}
+        self.bundles = {}  # type: Dict[Text, list]
         self.name_counter = 1
-        self.names_to_values = {}
+        self.names_to_values = {}  # type: Dict[Text, Any]
         self.__stream = CUnicodeIO()
         self.__printer = RepresentationPrinter(self.__stream)
+        self._initialize_rules_to_run = copy(self.initialize_rules())
 
     def __pretty(self, value):
+        if isinstance(value, VarReference):
+            return value.name
         self.__stream.seek(0)
         self.__stream.truncate(0)
         self.__printer.output_width = 0
@@ -472,6 +546,23 @@ class RuleBasedStateMachine(GenericStateMachine):
     def bundle(self, name):
         return self.bundles.setdefault(name, [])
 
+    @classmethod
+    def initialize_rules(cls):
+        try:
+            return cls._initializers_per_class[cls]
+        except KeyError:
+            pass
+
+        for k, v in inspect.getmembers(cls):
+            r = getattr(v, INITIALIZE_RULE_MARKER, None)
+            if r is not None:
+                cls.define_initialize_rule(
+                    r.targets, r.function, r.arguments, r.precondition,
+                )
+        cls._initializers_per_class[cls] = \
+            cls._base_initializers_per_class.pop(cls, [])
+        return cls._initializers_per_class[cls]
+
     @classmethod
     def rules(cls):
         try:
@@ -503,6 +594,23 @@ class RuleBasedStateMachine(GenericStateMachine):
         cls._invariants_per_class[cls] = target
         return cls._invariants_per_class[cls]
 
+    @classmethod
+    def define_initialize_rule(
+            cls, targets, function, arguments, precondition=None):
+        converted_arguments = {}
+        for k, v in arguments.items():
+            converted_arguments[k] = v
+        if cls in cls._initializers_per_class:
+            target = cls._initializers_per_class[cls]
+        else:
+            target = cls._base_initializers_per_class.setdefault(cls, [])
+
+        return target.append(
+            Rule(
+                targets, function, converted_arguments, precondition,
+            )
+        )
+
     @classmethod
     def define_rule(cls, targets, function, arguments, precondition=None):
         converted_arguments = {}
@@ -520,6 +628,14 @@ class RuleBasedStateMachine(GenericStateMachine):
         )
 
     def steps(self):
+        # Pick initialize rules first
+        if self._initialize_rules_to_run:
+            return one_of([
+                tuples(just(rule), fixed_dictionaries(rule.arguments))
+                for rule in self._initialize_rules_to_run
+            ])
+
+        # All initialize rules has been run once, go with the regular rules
         strategies = []
         for rule in self.rules():
             converted_arguments = {}
@@ -532,6 +648,7 @@ class RuleBasedStateMachine(GenericStateMachine):
                     if not bundle:
                         valid = False
                         break
+                    v = BundleReferenceStrategy(v.name)
                 converted_arguments[k] = v
             if valid:
                 strategies.append(tuples(
@@ -542,47 +659,43 @@ class RuleBasedStateMachine(GenericStateMachine):
                 u'No progress can be made from state %r' % (self,)
             )
 
-        for name, bundle in self.bundles.items():
-            if len(bundle) > 1:
-                strategies.append(
-                    builds(
-                        ShuffleBundle, just(name),
-                        lists(integers(0, len(bundle) - 1))))
-
         return one_of(strategies)
 
+    def print_start(self):
+        report(u'state = %s()' % (self.__class__.__name__,))
+
+    def print_end(self):
+        report(u'state.teardown()')
+
     def print_step(self, step):
-        if isinstance(step, ShuffleBundle):
-            return
         rule, data = step
         data_repr = {}
         for k, v in data.items():
             data_repr[k] = self.__pretty(v)
         self.step_count = getattr(self, u'step_count', 0) + 1
-        report(u'Step #%d: %s%s(%s)' % (
-            self.step_count,
+        report(u'%sstate.%s(%s)' % (
             u'%s = ' % (self.upcoming_name(),) if rule.targets else u'',
             rule.function.__name__,
             u', '.join(u'%s=%s' % kv for kv in data_repr.items())
         ))
 
     def execute_step(self, step):
-        if isinstance(step, ShuffleBundle):
-            bundle = self.bundle(step.bundle)
-            for i in step.swaps:
-                bundle.insert(i, bundle.pop())
-            return
         rule, data = step
         data = dict(data)
+        for k, v in list(data.items()):
+            if isinstance(v, VarReference):
+                data[k] = self.names_to_values[v.name]
         result = rule.function(self, **data)
         if rule.targets:
             name = self.new_name()
             self.names_to_values[name] = result
             self.__printer.singleton_pprinters.setdefault(
-                id(result), lambda obj, p, cycle: p.text(name),
+                id(result), lambda obj, p, cycle: p.text(name)
             )
             for target in rule.targets:
                 self.bundle(target).append(VarReference(name))
+        if self._initialize_rules_to_run:
+            self._initialize_rules_to_run.remove(rule)
 
     def check_invariants(self):
         for invar in self.invariants():
