@@ -20,6 +20,7 @@ from __future__ import division, print_function, absolute_import
 import enum
 import math
 import random
+import string
 import datetime as dt
 import operator
 from uuid import UUID
@@ -28,6 +29,8 @@ from inspect import isclass, isfunction
 from fractions import Fraction
 from functools import reduce
 
+import attr
+
 from hypothesis.errors import InvalidArgument, ResolutionFailed
 from hypothesis.control import note, assume, cleanup, current_build_context
 from hypothesis._settings import note_deprecation
@@ -79,14 +82,13 @@ if False:
     from random import Random  # noqa
     from typing import Any, Dict, Union, Sequence, Callable, Pattern  # noqa
     from typing import TypeVar, Tuple, List, Set, FrozenSet, overload  # noqa
-    from typing import Type, Mapping, Text, AnyStr, Optional  # noqa
+    from typing import Type, Text, AnyStr, Optional  # noqa
 
     from hypothesis.utils.conventions import InferType  # noqa
     from hypothesis.searchstrategy.strategies import T, Ex  # noqa
     K, V = TypeVar['K'], TypeVar['V']
     # See https://github.com/python/mypy/issues/3186 - numbers.Real is wrong!
     Real = Union[int, float, Fraction, Decimal]
-    ExtendFunc = Callable[[SearchStrategy[Union[T, Ex]]], SearchStrategy[T]]
 else:
     def overload(f):
         return f
@@ -108,7 +110,7 @@ __all__ = [
     'recursive', 'composite',
     'shared', 'runner', 'data',
     'deferred',
-    'from_type', 'register_type_strategy',
+    'from_type', 'register_type_strategy', 'emails',
 ]
 
 _strategies = set()
@@ -259,10 +261,23 @@ def none():
     return just(None)
 
 
-def one_of(
-    *args  # type: Union[SearchStrategy[Ex], Sequence[SearchStrategy[Ex]]]
-):
-    # type: (...) -> SearchStrategy[Ex]
+@overload
+def one_of(args):
+    # type: (Sequence[SearchStrategy[Any]]) -> SearchStrategy[Any]
+    pass  # pragma: no cover
+
+
+@overload
+def one_of(*args):
+    # type: (SearchStrategy[Any]) -> SearchStrategy[Any]
+    pass  # pragma: no cover
+
+
+def one_of(*args):
+    # Mypy workaround alert:  Any is too loose above; the return paramater
+    # should be the union of the input parameters.  Unfortunately, Mypy <=0.600
+    # raises errors due to incompatible inputs instead.  See #1270 for links.
+    # v0.610 doesn't error; it gets inference wrong for 2+ arguments instead.
     """Return a strategy which generates values from any of the argument
     strategies.
 
@@ -703,7 +718,7 @@ def iterables(elements=None, min_size=None, average_size=None, max_size=None,
 
 @defines_strategy
 def fixed_dictionaries(
-    mapping  # type: Mapping[T, SearchStrategy[Ex]]
+    mapping  # type: Dict[T, SearchStrategy[Ex]]
 ):
     # type: (...) -> SearchStrategy[Dict[T, Ex]]
     """Generates a dictionary of the same type as mapping with a fixed set of
@@ -728,12 +743,14 @@ def fixed_dictionaries(
 def dictionaries(
     keys,  # type: SearchStrategy[Ex]
     values,  # type: SearchStrategy[T]
-    dict_class=dict,  # type: Type[Mapping]
+    dict_class=dict,  # type: type
     min_size=None,  # type: int
     average_size=None,  # type: int
     max_size=None,  # type: int
 ):
-    # type: (...) -> SearchStrategy[Mapping[Ex, T]]
+    # type: (...) -> SearchStrategy[Dict[Ex, T]]
+    # Describing the exact dict_class to Mypy drops the key and value types,
+    # so we report Dict[K, V] instead of Mapping[Any, Any] for now.  Sorry!
     """Generates dictionaries of type dict_class with keys drawn from the keys
     argument and values drawn from the values argument.
 
@@ -874,7 +891,7 @@ def characters(
 @cacheable
 @defines_strategy_with_reusable_values
 def text(
-    alphabet=None,  # type: Union[Text, SearchStrategy[Text]]
+    alphabet=None,  # type: Union[Sequence[Text], SearchStrategy[Text]]
     min_size=None,   # type: int
     average_size=None,   # type: int
     max_size=None  # type: int
@@ -1047,6 +1064,10 @@ def builds(
     value :const:`hypothesis.infer` as a keyword argument to
     builds, instead of a strategy for that argument to the callable.
 
+    If the callable is a class defined with :pypi:`attrs`, missing required
+    arguments will be inferred from the attribute on a best-effort basis,
+    e.g. by checking :ref:`attrs standard validators <attrs:api_validators>`.
+
     Examples from this strategy shrink by shrinking the argument values to
     the callable.
     """
@@ -1071,16 +1092,22 @@ def builds(
         # Avoid an implementation nightmare juggling tuples and worse things
         raise InvalidArgument('infer was passed as a positional argument to '
                               'builds(), but is only allowed as a keyword arg')
-    hints = get_type_hints(target.__init__ if isclass(target) else target)
-    for kw in [k for k, v in kwargs.items() if v is infer]:
-        if kw not in hints:
-            raise InvalidArgument(
-                'passed %s=infer for %s, but %s has no type annotation'
-                % (kw, target.__name__, kw))
-        kwargs[kw] = from_type(hints[kw])
     required = required_args(target, args, kwargs)
-    for ms in set(hints) & (required or set()):
-        kwargs[ms] = from_type(hints[ms])
+    to_infer = set(k for k, v in kwargs.items() if v is infer)
+    if required or to_infer:
+        if isclass(target) and attr.has(target):
+            # Use our custom introspection for attrs classes
+            from hypothesis.searchstrategy.attrs import from_attrs
+            return from_attrs(target, args, kwargs, required | to_infer)
+        # Otherwise, try using type hints
+        hints = get_type_hints(
+            target.__init__ if isclass(target) else target)
+        if to_infer - set(hints):
+            raise InvalidArgument(
+                'passed infer for %s, but there is no type annotation'
+                % (', '.join(sorted(to_infer - set(hints)))))
+        for kw in set(hints) & (required | to_infer):
+            kwargs[kw] = from_type(hints[kw])
     # Mypy doesn't realise that `infer` is gone from kwargs now
     kwarg_strat = fixed_dictionaries(kwargs)  # type: ignore
     return tuples(tuples(*args), kwarg_strat).map(
@@ -1205,10 +1232,12 @@ def from_type(thing):
     if issubclass(thing, enum.Enum):
         assert len(thing), repr(thing) + ' has no members to sample'
         return sampled_from(thing)
-    # If the constructor has an annotation for every required argument,
-    # we can (and do) use builds() without supplying additional arguments.
+    # If the constructor has an annotation for all required arguments,
+    # or the class was defined with attrs, builds(thing) will work.
+    # (unless inference fails for an argument, but that's the same above too)
     required = required_args(thing)
-    if not required or required.issubset(get_type_hints(thing.__init__)):
+    if not required or required.issubset(get_type_hints(thing.__init__)) or \
+            attr.has(thing):
         return builds(thing)
     # We have utterly failed, and might as well say so now.
     raise ResolutionFailed('Could not resolve %r to a strategy; consider '
@@ -1439,7 +1468,7 @@ def decimals(
 
 def recursive(
     base,  # type: SearchStrategy[Ex]
-    extend,  # type: ExtendFunc
+    extend,  # type: Callable[[SearchStrategy[Any]], SearchStrategy[T]]
     max_leaves=100,  # type: int
 ):
     # type: (...) -> SearchStrategy[Union[T, Ex]]
@@ -1533,6 +1562,10 @@ def datetimes(
     package, but provides all timezones in the Olsen database.  If you want to
     allow naive datetimes, combine strategies like ``none() | timezones()``.
 
+    :py:func:`hypothesis.extra.dateutil.timezones` requires the
+    :pypi:`python-dateutil` package, and similarly provides all timezones
+    there.
+
     Alternatively, you can create a list of the timezones you wish to allow
     (e.g. from the standard library, ``datetutil``, or ``pytz``) and use
     :py:func:`sampled_from`.  Ensure that simple values such as None or UTC
@@ -2058,4 +2091,22 @@ def deferred(definition):
     return DeferredStrategy(definition)
 
 
+@defines_strategy_with_reusable_values
+def emails():
+    """A strategy for generating email addresses as unicode strings. The
+    address format is specific in :rfc:`5322#section-3.4.1`. Values shrink
+    towards shorter local-parts and host domains.
+
+    This strategy is useful for generating "user data" for tests, as
+    mishandling of email addresses is a common source of bugs. Future
+    updates will generate more complicated addresses allowed by the RFC.
+    """
+    from hypothesis.provisional import domains
+    local_chars = string.ascii_letters + string.digits + "!#$%&'*+-/=^_`{|}~"
+    local_part = text(local_chars, min_size=1, max_size=64)
+    # TODO: include dot-atoms, quoted strings, escaped chars, etc in local part
+    return builds(u'{}@{}'.format, local_part, domains()).filter(
+        lambda addr: len(addr) <= 255)
+
+
 assert _strategies.issubset(set(__all__)), _strategies - set(__all__)
